package sun.security.pkcs;

import sun.security.util.DerEncoder;
import sun.security.util.DerOutputStream;
import java.io.OutputStream;
import sun.security.util.DerValue;
import java.io.IOException;
import sun.security.util.DerInputStream;
import sun.security.util.ObjectIdentifier;
import java.util.Hashtable;

public class PKCS9Attributes
{
    private final Hashtable<ObjectIdentifier, PKCS9Attribute> attributes;
    private final Hashtable<ObjectIdentifier, ObjectIdentifier> permittedAttributes;
    private final byte[] derEncoding;
    private boolean ignoreUnsupportedAttributes;
    
    public PKCS9Attributes(ObjectIdentifier[] array, DerInputStream derInputStream) throws IOException {
        this.attributes = new Hashtable(3);
        this.ignoreUnsupportedAttributes = false;
        if (array != null) {
            this.permittedAttributes = new Hashtable(array.length);
            for (int i = 0; i < array.length; ++i) {
                this.permittedAttributes.put(array[i], array[i]);
            }
        }
        else {
            this.permittedAttributes = null;
        }
        this.derEncoding = this.decode(derInputStream);
    }
    
    public PKCS9Attributes(DerInputStream derInputStream) throws IOException {
        this(derInputStream, false);
    }
    
    public PKCS9Attributes(DerInputStream derInputStream, boolean ignoreUnsupportedAttributes) throws IOException {
        this.attributes = new Hashtable(3);
        this.ignoreUnsupportedAttributes = false;
        this.ignoreUnsupportedAttributes = ignoreUnsupportedAttributes;
        this.derEncoding = this.decode(derInputStream);
        this.permittedAttributes = null;
    }
    
    public PKCS9Attributes(PKCS9Attribute[] array) throws IllegalArgumentException, IOException {
        this.attributes = new Hashtable(3);
        this.ignoreUnsupportedAttributes = false;
        int i = 0;
        while (i < array.length) {
            ObjectIdentifier oid = array[i].getOID();
            if (this.attributes.containsKey(oid)) {
                throw new IllegalArgumentException("PKCSAttribute " + array[i].getOID() + " duplicated while constructing PKCS9Attributes.");
            }
            this.attributes.put(oid, array[i]);
            ++i;
        }
        this.derEncoding = this.generateDerEncoding();
        this.permittedAttributes = null;
    }
    
    private byte[] decode(DerInputStream derInputStream) throws IOException {
        byte[] byteArray = derInputStream.getDerValue().toByteArray();
        byteArray[0] = 49;
        DerValue[] set = new DerInputStream(byteArray).getSet(3, true);
        boolean n = true;
        for (int i = 0; i < set.length; ++i) {
            Object value;
            try {
                value = new PKCS9Attribute(set[i]);
            }
            catch (ParsingException ex) {
                if (this.ignoreUnsupportedAttributes) {
                    n = false;
                    continue;
                }
                throw ex;
            }
            ObjectIdentifier oid = value.getOID();
            if (this.attributes.get(oid) != null) {
                throw new IOException("Duplicate PKCS9 attribute: " + oid);
            }
            if (this.permittedAttributes != null && !this.permittedAttributes.containsKey(oid)) {
                throw new IOException("Attribute " + oid + " not permitted in this attribute set");
            }
            this.attributes.put(oid, value);
        }
        return n ? byteArray : this.generateDerEncoding();
    }
    
    public void encode(byte b, OutputStream outputStream) throws IOException {
        outputStream.write((int)b);
        outputStream.write(this.derEncoding, 1, this.derEncoding.length - 1);
    }
    
    private byte[] generateDerEncoding() throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        derOutputStream.putOrderedSetOf((byte)49, castToDerEncoder(this.attributes.values().toArray()));
        return derOutputStream.toByteArray();
    }
    
    public byte[] getDerEncoding() throws IOException {
        return (byte[])this.derEncoding.clone();
    }
    
    public PKCS9Attribute getAttribute(ObjectIdentifier key) {
        return (PKCS9Attribute)this.attributes.get(key);
    }
    
    public PKCS9Attribute getAttribute(String s) {
        return (PKCS9Attribute)this.attributes.get(PKCS9Attribute.getOID(s));
    }
    
    public PKCS9Attribute[] getAttributes() {
        PKCS9Attribute[] array = new PKCS9Attribute[this.attributes.size()];
        for (int n = 0, n2 = 1; n2 < PKCS9Attribute.PKCS9_OIDS.length && n < array.length; ++n2) {
            array[n] = this.getAttribute(PKCS9Attribute.PKCS9_OIDS[n2]);
            if (array[n] != null) {
                ++n;
            }
        }
        return array;
    }
    
    public Object getAttributeValue(ObjectIdentifier obj) throws IOException {
        try {
            return this.getAttribute(obj).getValue();
        }
        catch (NullPointerException ex) {
            throw new IOException("No value found for attribute " + obj);
        }
    }
    
    public Object getAttributeValue(String str) throws IOException {
        ObjectIdentifier oid = PKCS9Attribute.getOID(str);
        if (oid == null) {
            throw new IOException("Attribute name " + str + " not recognized or not supported.");
        }
        return this.getAttributeValue(oid);
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer(200);
        sb.append("PKCS9 Attributes: [\n\t");
        int n = 1;
        for (int i = 1; i < PKCS9Attribute.PKCS9_OIDS.length; ++i) {
            PKCS9Attribute attribute = this.getAttribute(PKCS9Attribute.PKCS9_OIDS[i]);
            if (attribute != null) {
                if (n != 0) {
                    n = 0;
                }
                else {
                    sb.append(";\n\t");
                }
                sb.append(attribute.toString());
            }
        }
        sb.append("\n\t] (end PKCS9 Attributes)");
        return sb.toString();
    }
    
    static DerEncoder[] castToDerEncoder(Object[] array) {
        DerEncoder[] array2 = new DerEncoder[array.length];
        for (int i = 0; i < array2.length; ++i) {
            array2[i] = (DerEncoder)array[i];
        }
        return array2;
    }
}
